@@ -47,8 +47,8 @@
 #include "gdict-about.h"
 #include "gdict-window.h"
 
-#define GDICT_WINDOW_COLUMNS      50
-#define GDICT_WINDOW_ROWS         30
+#define GDICT_WINDOW_COLUMNS      56
+#define GDICT_WINDOW_ROWS         33
 
 #define GDICT_WINDOW_MIN_WIDTH	  400
 #define GDICT_WINDOW_MIN_HEIGHT	  330
@@ -60,6 +60,7 @@ enum
   PROP_SOURCE_LOADER,
   PROP_SOURCE_NAME,
   PROP_PRINT_FONT,
+  PROP_DEFBOX_FONT,
   PROP_WORD,
   PROP_WINDOW_ID
 };
@@ -118,9 +119,15 @@ static void
 gdict_window_finalize (GObject *object)
 {
   GdictWindow *window = GDICT_WINDOW (object);
+
+  if (window->notify_id)
+    gconf_client_notify_remove (window->gconf_client, window->notify_id);
+
+  if (window->font_notify_id)
+    gconf_client_notify_remove (window->gconf_client, window->font_notify_id);
   
-  if (window->client)
-    g_object_unref (window->client);
+  if (window->gconf_client)
+    g_object_unref (window->gconf_client);
   
   if (window->context)
     {
@@ -144,6 +151,7 @@ gdict_window_finalize (GObject *object)
 
   g_free (window->source_name);
   g_free (window->print_font);
+  g_free (window->defbox_font);
   g_free (window->word);
   g_free (window->database);
   g_free (window->strategy);
@@ -206,6 +214,8 @@ gdict_window_error_cb (GdictContext *context,
 		       const GError *error,
 		       GdictWindow  *window)
 {
+  gdk_window_set_cursor (GTK_WIDGET (window)->window, NULL);
+
   gtk_statusbar_push (GTK_STATUSBAR (window->status), 0,
 		      _("No definitions found"));
 }
@@ -218,7 +228,7 @@ gdict_window_set_database (GdictWindow *window,
     g_free (window->database);
 
   if (!database)
-    database = gconf_client_get_string (window->client,
+    database = gconf_client_get_string (window->gconf_client,
 		    			GDICT_GCONF_DATABASE_KEY,
 					NULL);
   
@@ -294,12 +304,36 @@ get_context_from_loader (GdictWindow *window)
   return retval;
 }
 
+static void
+gdict_window_set_defbox_font (GdictWindow *window,
+			      const gchar *defbox_font)
+{
+  if (!defbox_font)
+    defbox_font = gconf_client_get_string (window->gconf_client,
+		    			   DOCUMENT_FONT_KEY,
+					   NULL);
+
+  if (!defbox_font)
+    defbox_font = GDICT_DEFAULT_DEFBOX_FONT;
+
+  if (window->defbox_font)
+    g_free (window->defbox_font);
+
+  window->defbox_font = g_strdup (defbox_font);
+
+  if (!window->defbox)
+    return;
+
+  gdict_defbox_set_font_name (GDICT_DEFBOX (window->defbox),
+		  	      window->defbox_font);
+}
+
 static void
 gdict_window_set_print_font (GdictWindow *window,
 			     const gchar *print_font)
 {
   if (!print_font)
-    print_font = gconf_client_get_string (window->client,
+    print_font = gconf_client_get_string (window->gconf_client,
     					  GDICT_GCONF_PRINT_FONT_KEY,
     					  NULL);
   
@@ -317,8 +351,10 @@ gdict_window_set_word (GdictWindow *window,
 		       const gchar *word)
 {
   g_free (window->word);
+  window->word = NULL;
 
-  window->word = g_strdup (word);
+  if (word && word[0] != '\0')
+    window->word = g_strdup (word);
 }
 
 static void
@@ -367,7 +403,7 @@ gdict_window_set_source_name (GdictWindow *window,
   
   /* some sanity checks first */
   if (!source_name)
-    source_name = gconf_client_get_string (window->client,
+    source_name = gconf_client_get_string (window->gconf_client,
       					   GDICT_GCONF_SOURCE_KEY,
       					   NULL);
   
@@ -409,6 +445,9 @@ gdict_window_set_property (GObject      *object,
     case PROP_PRINT_FONT:
       gdict_window_set_print_font (window, g_value_get_string (value));
       break;
+    case PROP_DEFBOX_FONT:
+      gdict_window_set_defbox_font (window, g_value_get_string (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -437,6 +476,9 @@ gdict_window_get_property (GObject    *object,
     case PROP_PRINT_FONT:
       g_value_set_string (value, window->print_font);
       break;
+    case PROP_DEFBOX_FONT:
+      g_value_set_string (value, window->defbox_font);
+      break;
     case PROP_WINDOW_ID:
       g_value_set_uint (value, window->window_id);
       break;
@@ -452,6 +494,22 @@ gdict_window_cmd_file_new (GtkAction   *action,
 {
   GtkWidget *new_window;
   
+  /* store the default size of the window and its state, so that
+   * it's picked up by the newly created window
+   */
+  gconf_client_set_int (window->gconf_client,
+		        GDICT_GCONF_WINDOW_WIDTH_KEY,
+		  	window->default_width,
+			NULL);
+  gconf_client_set_int (window->gconf_client,
+		  	GDICT_GCONF_WINDOW_HEIGHT_KEY,
+			window->default_height,
+			NULL);
+  gconf_client_set_bool (window->gconf_client,
+		  	 GDICT_GCONF_WINDOW_IS_MAXIMIZED_KEY,
+			 window->is_maximized,
+			 NULL);
+ 
   new_window = gdict_window_new (window->loader, NULL, NULL);
   gtk_widget_show (new_window);
   
@@ -530,8 +588,27 @@ gdict_window_cmd_file_close_window (GtkAction   *action,
 				    GdictWindow *window)
 {
   g_assert (GDICT_IS_WINDOW (window));
+
+  /* store the default size of the window and its state */
+  gconf_client_set_int (window->gconf_client,
+		        GDICT_GCONF_WINDOW_WIDTH_KEY,
+		  	window->default_width,
+			NULL);
+  gconf_client_set_int (window->gconf_client,
+		  	GDICT_GCONF_WINDOW_HEIGHT_KEY,
+			window->default_height,
+			NULL);
+  gconf_client_set_bool (window->gconf_client,
+		  	 GDICT_GCONF_WINDOW_IS_MAXIMIZED_KEY,
+			 window->is_maximized,
+			 NULL);
   
-  gtk_widget_destroy (GTK_WIDGET (window));
+  /* if this was called from the uimanager, destroy the widget;
+   * otherwise, if it was called from the delete_event, it will
+   * destroy the widget itself.
+   */
+  if (action)
+    gtk_widget_destroy (GTK_WIDGET (window));
 }
 
 static void
@@ -755,13 +832,40 @@ static const GtkActionEntry entries[] =
   { "Slash", GTK_STOCK_FIND, NULL, "slash", NULL, G_CALLBACK (gdict_window_cmd_edit_find) },
 };
 
+static gboolean
+gdict_window_delete_event_cb (GtkWidget *widget,
+			      GdkEvent  *event,
+			      gpointer   user_data)
+{
+  gdict_window_cmd_file_close_window (NULL, GDICT_WINDOW (widget));
+
+  return FALSE;
+}
+
+static gboolean
+gdict_window_state_event_cb (GtkWidget           *widget,
+			     GdkEventWindowState *event,
+			     gpointer             user_data)
+{
+  GdictWindow *window = GDICT_WINDOW (widget);
+  
+  if (event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED)
+    window->is_maximized = TRUE;
+  else
+    window->is_maximized = FALSE;
+  
+  return FALSE;
+}
+
 static void
 gdict_window_gconf_notify_cb (GConfClient *client,
 			      guint        cnxn_id,
 			      GConfEntry  *entry,
 			      gpointer     user_data)
 {
-  GdictWindow *window = GDICT_WINDOW (user_data);
+  GdictWindow *window;
+
+  window = GDICT_WINDOW (user_data);
 
   if (strcmp (entry->key, GDICT_GCONF_PRINT_FONT_KEY) == 0)
     {
@@ -784,6 +888,13 @@ gdict_window_gconf_notify_cb (GConfClient *client,
       else
         gdict_window_set_database (window, GDICT_DEFAULT_DATABASE);
     }
+  else if (strcmp (entry->key, DOCUMENT_FONT_KEY) == 0)
+    {
+      if (entry->value && (entry->value->type == GCONF_VALUE_STRING))
+        gdict_window_set_defbox_font (window, gconf_value_get_string (entry->value));
+      else
+        gdict_window_set_defbox_font (window, GDICT_DEFAULT_DEFBOX_FONT);
+    }
 }
 
 static void
@@ -854,27 +965,100 @@ gdict_window_drag_data_received_cb (GtkWidget        *widget,
 }
 
 static void
-gdict_window_style_set (GtkWidget *widget,
-			GtkStyle  *old_style)
+gdict_window_size_allocate (GtkWidget     *widget,
+			    GtkAllocation *allocation)
 {
-  PangoContext *pango_context;
-  PangoFontDescription *font_desc;
-  gint font_size, width, height;
+  GdictWindow *window = GDICT_WINDOW (widget);
 
-  if (GTK_WIDGET_CLASS (gdict_window_parent_class)->style_set)
-    GTK_WIDGET_CLASS (gdict_window_parent_class)->style_set (widget, old_style);
+  if (!window->is_maximized)
+    {
+      window->default_width = allocation->width;
+      window->default_height = allocation->height;
+    }
+
+  if (GTK_WIDGET_CLASS (gdict_window_parent_class)->size_allocate)
+    GTK_WIDGET_CLASS (gdict_window_parent_class)->size_allocate (widget,
+		    						 allocation);
+}
+
+static void
+set_window_default_size (GdictWindow *window)
+{
+  GtkWidget *widget;
+  gboolean is_maximized;
+  gint width, height;
+  gint font_size;
+  GdkScreen *screen;
+  gint monitor_num;
+  GtkRequisition req;
+  GdkRectangle monitor;
+
+  g_assert (GDICT_IS_WINDOW (window));
+
+  widget = GTK_WIDGET (window);
   
+  /* recover the state from GConf */
+  width = gconf_client_get_int (window->gconf_client,
+		  		GDICT_GCONF_WINDOW_WIDTH_KEY,
+				NULL);
+  height = gconf_client_get_int (window->gconf_client,
+		  		 GDICT_GCONF_WINDOW_HEIGHT_KEY,
+				 NULL);
+  is_maximized = gconf_client_get_bool (window->gconf_client,
+		  			GDICT_GCONF_WINDOW_IS_MAXIMIZED_KEY,
+					NULL);
   
-  pango_context = gtk_widget_get_pango_context (widget);
-  font_desc = pango_context_get_font_description (pango_context);
-  font_size = pango_font_description_get_size (font_desc) / PANGO_SCALE;
-  
-  width = MAX (GDICT_WINDOW_COLUMNS * font_size, GDICT_WINDOW_MIN_WIDTH);
-  height = MAX (GDICT_WINDOW_ROWS * font_size, GDICT_WINDOW_MIN_HEIGHT);
-  
+  /* XXX - the user wants gnome-dictionary to resize itself, so
+   * we compute the minimum safe geometry needed for displaying
+   * the text returned by a dictionary server, which is based
+   * on the font size and the ANSI terminal.  this is dumb,
+   * I know, but dictionary servers return pre-formatted text
+   * and we can't reformat it ourselves.
+   */
+  if (width == -1 || height == -1)
+    {
+      /* Size based on the font size */
+      GtkWidget *defbox = window->defbox;
+      
+      font_size = pango_font_description_get_size (defbox->style->font_desc);
+      font_size = PANGO_PIXELS (font_size);
+
+      width = font_size * GDICT_WINDOW_COLUMNS;
+      height = font_size * GDICT_WINDOW_ROWS;
+
+      /* Use at least the requisition size of the window... */
+      gtk_widget_size_request (widget, &req);
+      width = MAX (width, req.width);
+      height = MAX (height, req.height);
+
+      /* ... but make it no larger than the monitor */
+      screen = gtk_widget_get_screen (widget);
+      monitor_num = gdk_screen_get_monitor_at_window (screen, widget->window);
+
+      gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
+      width = MIN (width, monitor.width * 3 / 4);
+      height = MIN (height, monitor.height * 3 / 4);
+    }
+
+  /* Set default size */
   gtk_window_set_default_size (GTK_WINDOW (widget),
   			       width,
   			       height);
+
+  if (is_maximized)
+    gtk_window_maximize (GTK_WINDOW (widget));
+}
+
+static void
+gdict_window_style_set (GtkWidget *widget,
+			GtkStyle  *old_style)
+{
+  GdictWindow *window;
+
+  if (GTK_WIDGET_CLASS (gdict_window_parent_class)->style_set)
+    GTK_WIDGET_CLASS (gdict_window_parent_class)->style_set (widget, old_style);
+
+  set_window_default_size (GDICT_WINDOW (widget));
 }
 
 static GObject *
@@ -884,14 +1068,15 @@ gdict_window_constructor (GType                  type,
 {
   GObject *object;
   GdictWindow *window;
-  PangoContext *pango_context;
-  PangoFontDescription *font_desc;
-  gint font_size, width, height;
+  gint width, height;
+  gboolean is_maximized;
   GtkWidget *hbox;
   GtkWidget *vbox;
   GtkWidget *label;
   GtkActionGroup *action_group;
   GtkAccelGroup *accel_group;
+  PangoFontDescription *font_desc;
+  gchar *font_name;
   GError *error;
   
   object = G_OBJECT_CLASS (gdict_window_parent_class)->constructor (type,
@@ -900,18 +1085,6 @@ gdict_window_constructor (GType                  type,
   window = GDICT_WINDOW (object);
   
   gtk_widget_push_composite_child ();
-
-  pango_context = gtk_widget_get_pango_context (GTK_WIDGET (window));
-  font_desc = pango_context_get_font_description (pango_context);
-  font_size = pango_font_description_get_size (font_desc) / PANGO_SCALE;
-  
-  width = MAX (GDICT_WINDOW_COLUMNS * font_size, GDICT_WINDOW_MIN_WIDTH);
-  height = MAX (GDICT_WINDOW_ROWS * font_size, GDICT_WINDOW_MIN_HEIGHT);
-  
-  gtk_window_set_title (GTK_WINDOW (window), _("Dictionary"));
-  gtk_window_set_default_size (GTK_WINDOW (window),
-  			       width,
-  			       height);
  
   window->main_box = gtk_vbox_new (FALSE, 0);
   gtk_container_add (GTK_CONTAINER (window), window->main_box);
@@ -988,7 +1161,66 @@ gdict_window_constructor (GType                  type,
   gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (window->status), TRUE);
   gtk_box_pack_end (GTK_BOX (window->main_box), window->status, FALSE, FALSE, 0);
   gtk_widget_show (window->status);
+
+  /* retrieve the font size from gconf */
+  font_name = gconf_client_get_string (window->gconf_client,
+		  		       DOCUMENT_FONT_KEY,
+				       NULL);
+  if (!font_name)
+    font_name = GDICT_DEFAULT_DEFBOX_FONT;
   
+  gdict_window_set_defbox_font (window, font_name);
+  font_desc = pango_font_description_from_string (font_name);
+  
+  /* retrieve the window state from gconf */
+  is_maximized = gconf_client_get_bool (window->gconf_client,
+		  			GDICT_GCONF_WINDOW_IS_MAXIMIZED_KEY,
+					NULL);
+
+  width = gconf_client_get_int (window->gconf_client,
+		  		GDICT_GCONF_WINDOW_WIDTH_KEY,
+				NULL);
+  height = gconf_client_get_int (window->gconf_client,
+		  		 GDICT_GCONF_WINDOW_HEIGHT_KEY,
+				 NULL);
+  
+  /* if the (width, height) tuple is not defined, use the font to
+   * calculate the right window geometry
+   */
+  if (width == -1 || height == -1)
+    {
+      gint font_size;
+  
+      font_size = pango_font_description_get_size (font_desc);
+      font_size = PANGO_PIXELS (font_size);
+
+      width = MAX (GDICT_WINDOW_COLUMNS * font_size, GDICT_WINDOW_MIN_WIDTH);
+      height = MAX (GDICT_WINDOW_ROWS * font_size, GDICT_WINDOW_MIN_HEIGHT);
+    }
+  else
+    {
+      window->default_width = width;
+      window->default_height = height;
+    }
+
+  pango_font_description_free (font_desc);
+  
+  window->is_maximized = is_maximized;
+  
+  gtk_window_set_title (GTK_WINDOW (window), _("Dictionary"));
+  gtk_window_set_default_size (GTK_WINDOW (window),
+  			       width,
+  			       height);
+  if (is_maximized)
+    gtk_window_maximize (GTK_WINDOW (window));
+  
+  g_signal_connect (window, "delete-event",
+		    G_CALLBACK (gdict_window_delete_event_cb),
+		    NULL);
+  g_signal_connect (window, "window-state-event",
+		    G_CALLBACK (gdict_window_state_event_cb),
+		    NULL);
+
   gtk_widget_pop_composite_child ();
   
   return object;
@@ -1006,6 +1238,7 @@ gdict_window_class_init (GdictWindowClass *klass)
   gobject_class->constructor = gdict_window_constructor;
 
   widget_class->style_set = gdict_window_style_set;
+  widget_class->size_allocate = gdict_window_size_allocate;
   
   g_object_class_install_property (gobject_class,
   				   PROP_SOURCE_LOADER,
@@ -1028,6 +1261,13 @@ gdict_window_class_init (GdictWindowClass *klass)
   				   			"The font name to be used when printing",
   				   			GDICT_DEFAULT_PRINT_FONT,
   				   			(G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT)));
+  g_object_class_install_property (gobject_class,
+		  		   PROP_DEFBOX_FONT,
+				   g_param_spec_string ("defbox-font",
+					   		"Defbox Font",
+							"The font name to be used by the defbox widget",
+							GDICT_DEFAULT_DEFBOX_FONT,
+							(G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT)));
   g_object_class_install_property (gobject_class,
 		  		   PROP_WORD,
 				   g_param_spec_string ("word",
@@ -1061,26 +1301,64 @@ gdict_window_init (GdictWindow *window)
 {
   gchar *icon_file;
   GdkPixbuf *icon;
-  GError *icon_error;
+  GError *icon_error, *gconf_error;
   
   window->loader = NULL;
   window->context = NULL;
-  
-  window->client = gconf_client_get_default ();
-  gconf_client_add_dir (window->client,
+
+  window->gconf_client = gconf_client_get_default ();
+
+  gconf_error = NULL;
+  gconf_client_add_dir (window->gconf_client,
   			GDICT_GCONF_DIR,
   			GCONF_CLIENT_PRELOAD_NONE,
-  			NULL);
-  window->notify_id = gconf_client_notify_add (window->client,
+  			&gconf_error);
+  if (gconf_error)
+    {
+      show_error_dialog (NULL,
+		         _("Unable to connect to GConf"),
+		         gconf_error->message);
+
+      g_error_free (gconf_error);
+      gconf_error = NULL;
+    }
+
+  window->notify_id = gconf_client_notify_add (window->gconf_client,
   					       GDICT_GCONF_DIR,
   					       gdict_window_gconf_notify_cb,
   					       window,
   					       NULL,
-  					       NULL);
+  					       &gconf_error);
+  if (gconf_error)
+    {
+      show_error_dialog (NULL,
+		         _("Unable to get notification for preferences"),
+		         gconf_error->message);
+
+      g_error_free (gconf_error);
+      gconf_error = NULL;
+    }
+
+  window->font_notify_id = gconf_client_notify_add (window->gconf_client,
+		  				    DOCUMENT_FONT_KEY,
+						    gdict_window_gconf_notify_cb,
+						    window,
+						    NULL,
+						    &gconf_error);
+  if (gconf_error)
+    {
+      show_error_dialog (NULL,
+		         _("Unable to get notification for the document font"),
+		         gconf_error->message);
+
+      g_error_free (gconf_error);
+      gconf_error = NULL;
+    }
   
   window->word = NULL;
   window->source_name = NULL;
   window->print_font = NULL;
+  window->defbox_font = NULL;
 
   window->database = NULL;
   window->strategy = NULL;
@@ -1093,7 +1371,7 @@ gdict_window_init (GdictWindow *window)
   icon = gdk_pixbuf_new_from_file (icon_file, &icon_error);
   if (icon_error)
     {
-      show_error_dialog (GTK_WINDOW (window),
+      show_error_dialog (NULL,
 		         _("Unable to load the application icon"),
 		         icon_error->message);
 
@@ -1106,6 +1384,10 @@ gdict_window_init (GdictWindow *window)
     }
 
   g_free (icon_file);
+
+  window->default_width = -1;
+  window->default_height = -1;
+  window->is_maximized = FALSE;
   
   window->window_id = (gulong) time (NULL);
 }
